package zero1hd.rhythmbullet.audio;

import java.io.IOException;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.utils.Base64Coder;
import com.badlogic.gdx.utils.Json;
import com.mpatric.mp3agic.ID3v1;
import com.mpatric.mp3agic.InvalidDataException;
import com.mpatric.mp3agic.Mp3File;
import com.mpatric.mp3agic.UnsupportedTagException;

public class MP3Metadata implements AudioMetadata {
	private String author = "N/A";
	private String length;
	private String genre;
	private String lastPlayed;
	private Texture albumCover;
	private FileHandle fileHandle;
	private RhythmBulletMetadata rbMetadata;
	private Json json;
	public MP3Metadata() {
		json = new Json();
		try {
			Mp3File mp3file = new Mp3File(fileHandle.file());
			long lenInSec = mp3file.getLengthInSeconds();
			int min = (int) (lenInSec/60);
			length = (lenInSec/60) + ":" + (lenInSec - (min*60));
			
			if (mp3file.hasId3v1Tag()) {
				ID3v1 tag = mp3file.getId3v1Tag();
				author = tag.getArtist();
				genre = tag.getGenreDescription();
			}
			
			
			if (mp3file.hasCustomTag()) {
				rbMetadata = json.fromJson(RhythmBulletMetadata.class, new String(Base64Coder.decode(new String(mp3file.getCustomTag()))));
			} else {
				rbMetadata = new RhythmBulletMetadata();
				mp3file.setCustomTag(Base64Coder.encodeString(json.toJson(rbMetadata)).getBytes());
			}
			
		} catch (UnsupportedTagException e) {
			Gdx.app.debug("MP3Metadata", "Tag not supported by MP3agic.");
		} catch (InvalidDataException e) {
			Gdx.app.debug("MP3Metadata", "This files metadata has errors.");
		} catch (IOException e) {
			Gdx.app.debug("MP3Metadata", "IO error while reading metadata.");
		}
	}
	
	@Override
	public void loadAlbumCover() {
		try {
			Mp3File mp3file = new Mp3File(fileHandle.file());
			if (mp3file.hasId3v2Tag()) {
				byte[] imageData = mp3file.getId3v2Tag().getAlbumImage();
				Pixmap pixmap = new Pixmap(imageData, 0, imageData.length);
				albumCover = new Texture(pixmap);
				pixmap.dispose();
			}
		} catch (UnsupportedTagException | InvalidDataException | IOException e) {
			e.printStackTrace();
		}
		
	}

	@Override
	public void unloadAlbumCover() {
		albumCover.dispose();
		albumCover = null;
	}

	@Override
	public String getAuthor() {
		return author;
	}

	@Override
	public String getLength() {
		return length;
	}
	
	@Override
	public String getGenre() {
		return genre;
	}
	
	@Override
	public Texture getAlbumCover() {
		return albumCover;
	}

	@Override
	public String getLastPlayed() {
		return lastPlayed;
	}

	@Override
	public FileHandle getFileHandle() {
		return fileHandle;
	}
	
	@Override
	public RhythmBulletMetadata getRBMetadata() {
		return rbMetadata;
	}
	
	@Override
	public void flushRBMetadata() {
		try {
			Mp3File mp3file = new Mp3File(fileHandle.file());
			mp3file.setCustomTag(Base64Coder.encodeString(json.toJson(rbMetadata)).getBytes());
		} catch (UnsupportedTagException | InvalidDataException | IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void dispose() {
		if (albumCover != null) {
			albumCover.dispose();
			albumCover = null;
			
		}
	}

}
